package eGuideDog::Browser;

use strict;
use warnings;
use vars qw(@ISA);
require eGuideDog::Browser::Widget;
@ISA = qw(eGuideDog::Browser::Widget);
use eGuideDog::Browser::Const;
use eGuideDog::Browser::HTML;
use eGuideDog::Browser::Tab;
use eGuideDog::Browser::Progress;
use eGuideDog::Basic;
use Curses; # This is for key mapping. It's not good to port it to Windows.

sub new;
sub speaker;
sub cookie_file;
sub current_tab;
sub open_page;
sub press;
sub screen;
sub speech;
sub state;
sub command;

sub new {
  my ($class, $url) = @_;

  my $self = {tabs => [eGuideDog::Browser::Tab->new()],
	      tab_index => 0,
	      speaker => undef,
	      homepage => 'help/help.html',
	      proxy => undef,
	      bookmarks => undef,
	      controls => undef,
	      command => '',
	      command_para => [],
	      cookie_file => "$ENV{HOME}/.e-guidedog/cookie",
	     };

  # set URL
  $url = $self->{homepage} if (!$url);
  $url = 'help/help.html' if (!$url);

  bless $self, __PACKAGE__;
  $self->open_page($url);
  return $self;
}

sub speaker {
  my ($self, $speaker) = @_;
  if ($speaker) {
    $self->{speaker} = $speaker;
  } else {
    return $self->{speaker};
  }
}

sub cookie_file {
  my $self = shift;
  return $self->{cookie_file};
}

sub current_tab {
  my $self = shift;
  return $self->{tabs}->[$self->{tab_index}];
}

sub open_page {
  my ($self, $url, $postdata) = @_;
  $url = get_full_path($url);

  # http/https protocol use progress bar
  if ($url =~ /^http/) {
    # make a progress bar indicating the status of page loading
    my $html = eGuideDog::Browser::HTML->new();
    $html->url($url);
    $html->browser($self);
    my $html_thread = sub { return $html->get_html($url, $postdata); };
    my $html_progress = sub {
      my $percent = $html->percent;
      if (defined $percent) {
	$percent .= ' ' . gettext('finished') . ', ';
      } else {
	$percent = '';
      }
      my $progress = $html->progress;
      $progress = 0 if (not defined $progress);
      $progress .= ' ' . gettext('loaded') . ', ';
      my $speed = $html->speed;
      if (defined $speed && $speed =~ /\d/) {
	$speed = gettext('at the speed of') . ' ' . $speed . ',';
      } else {
	$speed = '';
      }
      my $eta = $html->eta;
      $eta = '' if (not defined $html->eta);
      return $percent . $progress . $speed . $eta;
    };
    my $callback = sub {
      my $content = shift;
      # $self = eGuideDog::Browser
      # $html = eGuideDog::Browser::HTML
      if ($$content) {
	$html->load($html->get_tree($content));
      } else {
	$url = get_full_path(gettext('help/page_not_found.html'));
	$html = eGuideDog::Browser::HTML->new($url);
	die "file $url not found!" unless $html;
      }
      $self->current_tab->current_control($html);
    };
    my $finished = sub { return $html->finished; };
    my $progress = eGuideDog::Browser::Progress->new($callback, $html_thread, gettext("Loading page $url"), $finished, $html_progress);
    $self->current_tab->current_control($progress);
  }

  # file protocol don't use progress bar
  elsif ($url =~ /^file:/) {
    my $html = eGuideDog::Browser::HTML->new($url);
    if (not $html) {
      $url = get_full_path(gettext('help/page_not_found.html'));
      $html = eGuideDog::Browser::HTML->new($url);
      die "file $url not found!" unless $html;
    }
    $html->browser($self);
    $self->current_tab->current_control($html);
  }
}

sub press {
  my ($self, $key) = @_;

  my $c = eGuideDog::Browser::Const::new(); # key reference

  # clear command
  if ($self->{command} && $self->{command} == $c->AUTO_SPEAK && $key ne '-1' && $key ne $c->KEYS_DOWN) {
    $self->{command} = undef;
}

  my $result = $self->current_tab->current_control->press($key);

#  if ($result == eGuideDog::Browser::Const::CANCEL) {
#    $self->current_tab->current_control = pop(@{$self->current_tab->{controls}});
#    $self->{screen} = $self->current_tab->current_control->{screen};
#    return eGuideDog::Browser::Const::LEGAL_KEY;
#  }

  # got key control
  if ($result == $c->THROW_KEY) {

    # quit
    if (lc($key) eq 'q' || $key eq $c->CTRL_Q) {
      #      $helper->help('quit_eGuideDog');
      return eGuideDog::Browser::Const::EXIT;
    }

    # continue speaking
    elsif (lc($key) eq 'a' || $key eq $c->CTRL_A) {
      $self->{command} = $c->AUTO_SPEAK;
    }

    # open url
    elsif (lc($key) eq 'o' || $key eq $c->CTRL_O) {
      my $callback = sub {
	# $self eGuideDog::Browser
	my $url = shift;
	$self->open_page($url);
      };
      my $input = eGuideDog::Browser::InputText->new($callback, 'Please enter URL:');
#      push(@{$self->current_tab->{controls}}, $self->current_tab->current_control);
#      $self->current_tab->current_control->detach();
      $self->current_tab->current_control($input);
    }

    # change encoding
    elsif (lc($key) eq 'e' || $key eq $c->CTRL_E) {
      my $html = $self->current_tab->current_control;
      my $callback = sub {
	# $self eGuideDog::Browser
	my $encoding = shift;
	$html->reload($encoding);
	$self->current_tab->current_control($html);
      };
      my $input = eGuideDog::Browser::InputText->new($callback, 'Please input new encoding for current page:');
      $self->current_tab->current_control($input);
    }

    # find string
    elsif (lc($key) eq 'f' || $key eq $c->CTRL_F) {
      my $html = $self->current_tab->current_control;
      my $callback = sub {
	# $self eGuideDog::Browser
	my $keyword = shift;
	my $result = $html->navigate->find($keyword);
	$html->screen->index($html->navigate->node_index);
	if ($result) {
	  $html->speechs->index($html->navigate->node_index);
	} else {
	  $html->speech("$keyword not found");
	}
	$html->screen->update(1);
	$self->current_tab->current_control($html);
      };
      my $input = eGuideDog::Browser::InputText->new($callback, 'Please input string to search:');
      $self->current_tab->current_control($input);
    }

    # stop speech(' ')
    elsif ($key eq ' ' || $key eq $c->ALT_SPACE) {
      $self->{speaker}->stop();
    }

    # speak current word & switch to word mode
    elsif ($key eq $c->ALT_w) {
      $self->{command} = $c->WORD_MODE;
    }

    # speak current char & switch to char mode
    elsif ($key eq $c->ALT_c) {
      $self->{command} = $c->CHAR_MODE;
    }

    # speak previous word/char
    elsif ($key eq 'alt + KEY_LEFT') {
    }

    # speak next word/char
    elsif ($key eq 'alt + KEY_RIGHT') {
    }

    # speak slower
    elsif ($key eq $c->ALT_COMMA) {
      $self->{speaker}->rate($self->{speaker}->rate() - 10);
    }

    # speak faster
    elsif ($key eq $c->ALT_FULLSTOP) {
      $self->{speaker}->rate($self->{speaker}->rate() + 10);
    }

    # set viriant voice
    elsif ($key eq $c->ALT_0) {
      $self->{speaker}->variant(0);
    } elsif ($key eq $c->ALT_1) {
      $self->{speaker}->variant(1);
    } elsif ($key eq $c->ALT_2) {
      $self->{speaker}->variant(2);
    } elsif ($key eq $c->ALT_3) {
      $self->{speaker}->variant(3);
    } elsif ($key eq $c->ALT_4) {
      $self->{speaker}->variant(4);
    } elsif ($key eq $c->ALT_5) {
      $self->{speaker}->variant(5);
    } elsif ($key eq $c->ALT_6) {
      $self->{speaker}->variant(6);
    } elsif ($key eq $c->ALT_7) {
      $self->{speaker}->variant(7);
    } elsif ($key eq $c->ALT_8) {
      $self->{speaker}->variant(8);
    } elsif ($key eq $c->ALT_9) {
      $self->{speaker}->variant(9);
    }

    # speak louder
    elsif ($key eq $c->ALT_EQUAL) {
      $self->{speaker}->volume($self->{speaker}->volume() - 10);
    }

    # speak lower
    elsif ($key eq $c->ALT_MINUS) {
      $self->{speaker}->volume($self->{speaker}->volume() + 10);
    }

    # mute
    elsif ($key eq $c->ALT_m) {
      $self->{speaker}->mute(1 - $self->{speaker}->mute());
      $self->{speaker}->stop();
    }

    # reset all (rate, volume, pitch, range, punctuation, capitals) to default
    elsif ($key eq $c->ALT_z) {
    }

    # set language
    elsif ($key eq $c->ALT_l) {
      my $html = $self->current_tab->current_control;
      my $callback = sub {
	# $self eGuideDog::Browser
	my $language = shift;
	$html->screen->update(1);
	$self->current_tab->current_control($html);
	$self->{speaker}->language($language);
      };
      my $input = eGuideDog::Browser::InputText->new($callback, 'Please input language you want to use:');
      $self->current_tab->current_control($input);
    }

    # lower pitch
    elsif ($key eq $c->ALT_LEFT_BRACKET) {
      $self->{speaker}->pitch($self->{speaker}->pitch() - 10);
    }

    # higher pitch
    elsif ($key eq $c->ALT_RIGHT_BRACKET) {
      $self->{speaker}->pitch($self->{speaker}->pitch() + 10);
    }

  } else {
    return $result;
  }
}

sub screen {
  my ($self) = @_;
  return $self->current_tab->current_control->screen;
}

sub speech {
  my $self = shift;
  return $self->current_tab->current_control->speech;
}

sub state {
  my $self = shift;
  return $self->current_tab->current_control->state;
}

sub command {
  my ($self, $command) = @_;

  if (defined $command) {
    $self->{command} = $command;
  } else {
    return $self->{command};
  }
}

1;
