{
package eGuideDog::Browser::Progress;

use strict;
use warnings;
use threads;
use eGuideDog::Browser::Const;
use eGuideDog::Basic;
use vars qw(@ISA);
require eGuideDog::Browser::Widget;
@ISA = qw(eGuideDog::Browser::Widget);

sub new;
sub progress;
sub press;
sub screen;

sub new {
  my ($class, $callback, $thread, $caption, $finished_sub, $progress_sub) = @_;

  my $self = $class->SUPER::new($callback);
  bless $self, __PACKAGE__;
  $self->{caption} = $caption;
  $self->{progress_sub} = $progress_sub;
  $self->{finished_sub} = $finished_sub;
  $self->{thread} = threads->create($thread);
  die "Cannot create thread" unless $self->{thread};
  return $self;
}

sub finished {
  my $self = shift;
  return &{$self->{finished_sub}}();
}

sub progress {
  my ($self) = @_;
  if ($self->{progress_sub}) {
    return &{$self->{progress_sub}}();
  } else {
    return gettext('Loading');
  }
}

sub screen {
  my ($self) = @_;
  if (not defined $self->{screen}) {
    my @buffer;
    push(@buffer, $self->{caption});
    push(@buffer, $self->progress);
    $self->{screen} = eGuideDog::Terminal::Buffer->new(\@buffer);
  } else {
    $self->{screen}->{lines}->[1] = $self->progress;
  }
  return $self->{screen};
}

sub press {
  my ($self, $key) = @_;

  $self->{screen}->update(1);
  if ($self->finished) {
    my $content = $self->{thread}->join;
    &{$self->{callback}}(\$content);
  } else {
    sleep(1);
  }

  return eGuideDog::Browser::Const::CATCH_KEY;
}
}
1;
